

/*
	Tims_Electronic_Deviders.ino
	By Tim Jackson.1960

	Creadits:
		Arduino.
		LiquidCrystal_I2C based on work by DFRobot.

	This is code for: Tim's Electronic Deviders.
	I am using 3D Printed Deviders with a 49E Linear Hall Effect Sensor and two Magnets.
	I am using an Arduino NANO to calculate the postion of the deviders from the values recived from the 49E Linear Hall Effect Sensor.
	The sensor is linear: 3 mV/GS, but to get the angle a Sine of the value is calculated.

	S=O/H C=A/H T=O/A
	Degrees to Radians = degrees * (PI / 180)
	Radians to Degrees = radians * 180.0 / pi

*/
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

#define Hall_49R_Pin					A1		//	define Hall Effect Pin
#define CAL_0							377		//	Value of hall efect sensor at 0 angle
#define CAL_180							690		//	Value of hall efect sensor at 180 angle
#define CAL_RANGE (CAL_180 - CAL_0)				//	Highest CAL - Lowest CAL
#define CAL_RAD ((double)CAL_RANGE / 2)			//	Used in the sine H value to get angle.
#define LEG_LENGTH 99							//	Length of Devider Legs.
#define MAG_BIOS						-3.11	//	Set to zero, then change to a value that will correct the mesurment about 60 Degrees.
double Angle = 0;								//	Variable to hold Angle value.
double Length = 0;								//	Variable to hold Length value.
int SensorValue = CAL_0 + 1;					//	Variable to hold Sensor value.

LiquidCrystal_I2C lcd(0x27, 16, 2);		//20 to	27	//	Set the LCD address to 0x27 for a 16 chars and 2 line display


void  setup()
{
	Serial.begin(115200);			//	Start Serial.
	pinMode(Hall_49R_Pin, INPUT);	//	Define as Input.

	lcd.init();						//	Start LCD.
	lcd.backlight();				//	Turn on back light.
	lcd.setCursor(0, 0);			//	Set Cursor at the begining of line 0 (Top Line).
	lcd.print(" Angle:");			//	Display Label for Angle on top line.
	lcd.setCursor(0, 1);			//	Set Cursor at the begining of line 1 (Bottom Line).
	lcd.print("Length:");			//	Display Label for Length on bottom line.

}
void  loop() {

	SensorValue = analogRead(Hall_49R_Pin);	//	Read the value from sensor.

	Serial.println(SensorValue);			//	Send value to serial.
	CalcAngle();							//	Do sub routeen for angle.
	CalcLength();							//	Do sub routeen for length.
	Serial.println();						//	Send a new line to serial to seperate values.

	delay(200);								//	Wait a little for things to happen.
}
/*
	Calculate the Angle using Value from Sensor.

		S=O/H
		Radians to Degrees = radians * 180.0 / pi

		O = SensorValue - CAL_0 - CAL_RAD
		H = CAL_RAD
		Angle in radians = asin(O / H)
		Angle in degrees = Angle in radians * 180.0 / pi;

*/
void CalcAngle() {

	double O = (double)SensorValue - CAL_0 - CAL_RAD;
	double H = (double)O / CAL_RAD;
	Angle = 90.0 + (asin(H) * 180.0 / PI);

	Serial.print("Angle ");
	Serial.println(Angle + MAG_BIOS, 4);

	lcd.setCursor(7, 0);
	lcd.print("    ");
	lcd.setCursor(Xpos(Angle + MAG_BIOS), 0);
	lcd.print(Angle + MAG_BIOS, 3);
	lcd.print("    ");

}
/*
	Calculate the Length using degrees calculated from Sensor.

		C=A/H
		A=C*H
		Degrees to Radians = degrees * (PI / 180)

		C = (180 - Angle in degrees) / 2
		H = LEG_LENGTH
		A = C * H
		Length = A * 2

*/
void CalcLength() {

	double _angle = (180.0 - Angle) / 2;
	double _rad = _angle * (PI / 180);
	double C = cos(_rad);
	double H = LEG_LENGTH;
	Length = C * H * 2;

	Serial.print("Length ");
	Serial.println(Length + MAG_BIOS, 4);

	lcd.setCursor(7, 1);
	lcd.print("    ");
	lcd.setCursor(Xpos(Length + MAG_BIOS), 1);
	lcd.print(Length + MAG_BIOS, 3);
	lcd.print("    ");

}
/*
	This a function to calculate the position of value displayed on LCD.

		Check to see if value is hundreds, tens or single.
		To keep numbers alighned.
*/
byte Xpos(byte number) {
	byte val = 8;
	if (number < 100) { val = 9; }
	if (number < 10) { val = 10; }
	return val;
}
